from Bio.motifs import create
from Bio.SeqIO import parse
from numpy.random import random
from tensorflow.io import TFRecordWriter
from tensorflow.data import TFRecordDataset
from tensorflow.train import BytesList, Example, Feature, Features, FloatList
from typing import List

from constants import TRAIN_DATASET, TEST_DATASET


def generate_example(sequence, weight_matrix) -> bytes:
    """
    Create a binary-string for each sequence containing the sequence and the bases' frequency
    """
    schema = {
        "sequence": Feature(bytes_list=BytesList(value=[sequence.encode()])),
        "A_counts": Feature(float_list=FloatList(value=[weight_matrix["A"][0]])),
        "C_counts": Feature(float_list=FloatList(value=[weight_matrix["C"][0]])),
        "G_counts": Feature(float_list=FloatList(value=[weight_matrix["G"][0]])),
        "T_counts": Feature(float_list=FloatList(value=[weight_matrix["T"][0]])),
    }
    example = Example(features=Features(feature=schema))
    return example.SerializeToString()


def parse_data(filepath) -> List[bytes]:
    """
    Parse a FASTQ file and generate a List of serialized Examples
    """
    examples = []
    with open(filepath) as handle:
        for row in parse(handle, "fastq"):
            sequence = str(row.seq)
            motifs = create(row.seq)
            example = generate_example(sequence=sequence, weight_matrix=motifs.pwm)
            examples.append(example)
    return examples


def create_dataset(filepath) -> None:
    """
    Create a training and test dataset with a 70/30 split respectively
    """
    data = parse_data(filepath)
    train_test_split = 0.7
    with TFRecordWriter(TRAIN_DATASET) as train, TFRecordWriter(TEST_DATASET) as test:
        for element in data:
            if random() < train_test_split:
                train.write(element)
            else:
                test.write(element)


def read_dataset():
    pass


create_dataset("data/curesim-HVR.fastq")
